%{
/*
 *  ippl.y - functions parsing the configuration file
 *
 *  Copyright (C) 1998 Etienne Bernard
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "configuration.h"
#include "filter.h"

void yyerror(char *);
int yylex(void);

extern struct filter_entry *filter;
extern unsigned short log_protocols;

%}

%union {
  long longval;
  char * stringval;
  struct portrange_struct rangeval;
  struct hostdesc_struct hostval;
  union loginfo_union loginfoval;
  struct filter_entry * entryval;
  struct {
    long protocol;
    union loginfo_union loginfoval;
  } protocolval;
}

%token<stringval> IP HOSTMASK
%token<longval>    NUMBER

%token RUN LOG IGNORE FROM TYPE PORT
%token ICMP TCP ALL

%token RANGE SLASH EOL

%type<entryval> Rule
%type<longval> Action
%type<rangeval> Range
%type<hostval> From
%type<protocolval> Protocol

%start Input
%%

Input:
          /* Empty */
        | Input Line
        ;

Line:
          EOL
        | RUN LogList EOL
        | Rule EOL
          { $1->next = filter; filter = $1; }
        ;

LogList:
          LogItem
        | LogList LogItem
        ;

LogItem:
          ICMP
          { log_protocols |= RUN_ICMP; }
        | TCP
          { log_protocols |= RUN_TCP;  }
        | ALL
          { log_protocols |= (RUN_TCP & RUN_ICMP); }
        ;

Rule:
          Action Protocol
          {
            $$ = (struct filter_entry *)malloc(sizeof(struct filter_entry));
            $$->log = $1;
            $$->protocol = $2.protocol;
            $$->loginfo = $2.loginfoval;
            $$->hostdesc.address = $$->hostdesc.mask = 0;
            $$->hostdesc.hostmask = NULL;
          }
        | Action Protocol FROM From
          {
            $$ = (struct filter_entry *)malloc(sizeof(struct filter_entry));
            $$->log = $1;
            $$->protocol = $2.protocol;
            $$->loginfo = $2.loginfoval;
            $$->hostdesc = $4;
          }
        ;

Action:
          LOG     { $$ = 1; }
        | IGNORE  { $$ = 0; }
        ;

Protocol:
          ICMP
          { $$.protocol = ICMP_PROTO; $$.loginfoval.icmptype = 0; }
        | ICMP TYPE NUMBER
          { $$.protocol = ICMP_PROTO; $$.loginfoval.icmptype = $3; }
        | TCP
          {
            $$.protocol = TCP_PROTO;
            $$.loginfoval.portrange.range_min = NOT_DEFINED;
            $$.loginfoval.portrange.range_max = NOT_DEFINED;
          }
        | TCP PORT Range
          { $$.protocol = TCP_PROTO; $$.loginfoval.portrange = $3; }
        ;

Range:
          NUMBER               { $$.range_min = $1; $$.range_max = $1; }
        | RANGE NUMBER         { $$.range_min = NOT_DEFINED;
                                 $$.range_max = $2;                    }
        | NUMBER RANGE         { $$.range_min = $1;
                                 $$.range_max = NOT_DEFINED;           }
        | NUMBER RANGE NUMBER  { $$.range_min = $1; $$.range_max = $3; }
        ;

From:
          IP
            {
              struct in_addr inp;
              inet_aton($1, &inp);
              $$.address = inp.s_addr;
              $$.mask = 0xffffffff;
              $$.hostmask = NULL;
              free($1);
            }
        | IP SLASH IP
            {
              struct in_addr inp;
              inet_aton($1, &inp);
              $$.address = inp.s_addr;
              inet_aton($3, &inp);
              $$.mask = inp.s_addr;;
              $$.hostmask = NULL;
              free($1); free($3);
            }
        | HOSTMASK
            {
              $$.hostmask = $1;
              $$.address = 0;
              $$.mask = 0;
            }
        ;

%%

void yyerror(char *s) {
  extern int line_number;
  printf("%s line %d\n",s, line_number);
}
