/******************************************************************************
 * IP protocols logger					                      *
 ******************************************************************************
 * Written by Hugo Haas (hugo@debian.org)                                     *
 * Based on the idea of Mike Edulla (medulla@infosoc.com)                     *
 * Copyright (C) 1998-1999 Hugo Haas                                          *
 ******************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <syslog.h>
#include <unistd.h>
#include <errno.h>
#include <pwd.h>
#include <stdlib.h>

#include "defines.h"
#include "configuration.h"
#include "icmp.h"
#include "tcp.h"
#include "log.h"

/* Do we have to run as a daemon? */
short run_as_daemon = TRUE;

/* Configuration file to use */
char *configuration_file = NULL;

/* Logging mechanism */
struct logger log = { NULL, NULL };

/* Protocols that we log */
unsigned short log_protocols;

/*
 * do_help
 *
 * Display the command line options
 */

void do_help() {

  printf("ippl version %s \n\n", VERSION);
  printf("\t-n, --nodaemon:\tdo not run as a daemon and write information to stdout\n");
  printf("\t-c, --config <file>:\tspecify an alternate configuration file\n");
  printf("\t-h, --help:\tdisplay this help and exit\n");
  printf("\nPlease report bugs to ippl@via.ecp.fr\n");
}

/*
 * run_thread
 *
 * Run a thread
 *
 */

void run_thread(void *function, void *param) {
  pthread_attr_t attr_t;
  pthread_t t;

  pthread_attr_init(&attr_t);
  pthread_attr_setdetachstate(&attr_t, PTHREAD_CREATE_DETACHED);
  pthread_create(&t, &attr_t, function, param);
  pthread_attr_destroy(&attr_t);
}

/*
 * parse_arguments
 *
 * Parse the arguments given on the command line
 */

void parse_arguments(char **argv) {

  argv++;
  while(*argv) {
    if (!strcmp(*argv, "-h") || !strcmp(*argv, "--help")) {
      do_help();
      exit(0);
    }
    else if (!strcmp(*argv,"-n") || !strcmp(*argv,"--nodaemon")) {
      run_as_daemon = FALSE;
    }
    else if (!strcmp(*argv,"-c") || !strcmp(*argv,"--config")) {
      if (!*++argv) {
	printf("Missing argument: please specify configuration file\n");
	do_help();
	exit(1);
      }
      configuration_file = *argv;
    }
    else {
      printf("Unknown option: %s\n\n", *argv);
      do_help();
      exit(1);
    }
    argv++;
  }
}

/*
 * go_background
 *
 * Run as a daemon
 *
 * Based on iplogger (C) Mike Edulla + modifications by Topi Miettinen
 */

void go_background() {
  int     fd;
  int     fs;

  if (getppid() != 1) {
    signal(SIGTTOU, SIG_IGN);
    signal(SIGTTIN, SIG_IGN);
    signal(SIGTSTP, SIG_IGN);
    fs = fork();
    if (fs < 0) {
      perror("fork");
      exit(1);
    }
    if (fs > 0)
      exit(0);
    fd = open("/dev/tty", O_RDWR);
    if (fd >= 0) {
      ioctl(fd, TIOCNOTTY, (char *) NULL);
      close(fd);
    }
  }
  for (fd = 0; fd < NOFILE; fd++)
    close(fd);
  errno = 0;
  /*  chdir("/"); */
  umask(0);
  setsid();
}

/*
 * MAIN
 */

int main(int argc, char **argv) {
  struct passwd *nobody;

  parse_arguments(argv);

  setuid(0);
  if (geteuid() != 0) {
    printf("You must be root to run this program.\n");
    exit(1);
  }

  nobody = getpwnam("nobody");
  if (!nobody) {
    printf("I cannot find the \"nobody\" account.\n");
    exit(1);
  }

  if (configuration_file == NULL)
    configuration_file = strdup(CONFIGURATION_FILE);

  parse_config_file(configuration_file);

  if (log_protocols == NONE) {
    printf("Nothing to log.\n");
    exit(2);
  }

  if (run_as_daemon) {
    if (log.file)
      log.f = (void *) filelog;
    else
      log.f = (void *) syslog;
    go_background();
  }
  else {
    log.f = (void *) stdoutlog;
  }

  if (log_protocols & RUN_ICMP) {
    if (log_protocols & RUN_TCP)
      run_thread((void *) log_icmp, (void *)nobody);
    else
      log_icmp((void *)nobody);
  }

  if (log_protocols & RUN_TCP)
    log_tcp((void *)nobody);

  free(nobody);

  exit(0);
}
